<?php
/**
 * Plugin Name: FooEvents Seating
 * Description: Set up seating charts for events
 * Version: 1.4.11
 * Author: FooEvents
 * Plugin URI: https://www.fooevents.com/
 * Author URI: https://www.fooevents.com/
 * Developer: FooEvents
 * Developer URI: https://www.fooevents.com/
 * Text Domain: fooevents-seating
 *
 * Copyright: © 2009-2020 FooEvents.
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

//include config
require(WP_PLUGIN_DIR.'/fooevents_seating/config.php');


class Fooevents_Seating {
    
    private $Config;
    private $UpdateHelper;
    
    public function __construct() {
        
        add_action('admin_notices', array($this, 'check_fooevents'));
        add_action('woocommerce_product_write_panel_tabs', array($this, 'add_product_seating_options_tab'), 25);
        add_action('woocommerce_product_data_panels', array($this, 'add_product_seating_options_tab_options'));
        add_action('wp_enqueue_scripts', array($this, 'register_scripts_frontend')); 
        add_action('admin_enqueue_scripts', array($this, 'register_scripts'));
        add_action('admin_enqueue_scripts', array($this, 'register_styles'));
        add_action('woocommerce_process_product_meta', array($this, 'process_meta_box'));
        add_action('plugins_loaded', array($this, 'load_text_domain'));
        add_action('save_post', array(&$this, 'save_ticket_meta_boxes'), 1, 2);
        add_action('woocommerce_before_add_to_cart_button', array($this, 'output_seating_fields_product'));
        add_action('admin_init', array($this, 'register_seating_options'));
        add_action('wp_ajax_fooevents_fetch_add_ticket_seating_options', array($this, 'fetch_add_ticket_seating_options'));
        add_action('wp_ajax_fooevents_refresh_seating_chart', array($this, 'refresh_seating_chart'));
        add_action('wp_ajax_fetch_all_woocommerce_variation_attributes', array($this, 'fetch_all_woocommerce_variation_attributes'));
        add_action('wp_ajax_nopriv_fetch_all_woocommerce_variation_attributes', array($this, 'fetch_all_woocommerce_variation_attributes'));
        add_action('wp_ajax_fetch_selected_variation_attributes', array($this, 'fetch_selected_variation_attributes'));
        add_action('wp_ajax_nopriv_fetch_selected_variation_attributes', array($this, 'fetch_selected_variation_attributes'));
        add_action('woocommerce_add_cart_item_data', array($this, 'add_seating_addtocart'), 10, 4);  
        add_action('woocommerce_product_duplicate', array($this, 'duplicate_seat_variation_options'), 10, 2);
        add_action('before_delete_post', array($this, 'remove_unavailable_seats_from_event'), 10, 2);
        

        $this->plugin_init();
        
    }

    /**
     * Initializes plugin
     * 
     */
    public function plugin_init() {
        
        //Main config
        $this->Config = new FooEvents_Seating_Config();
        
        //UpdateHelper
        require_once($this->Config->classPath.'updatehelper.php');
        $this->UpdateHelper = new Fooevents_Seating_Update_Helper($this->Config);
        
        if( !function_exists('get_plugin_data') ){
            
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            
        }
        
        $this->Config->PluginData = get_plugin_data( __FILE__ );
        
    }
   
    /**
     * Registers scripts on the Wordpress frontend.
     * 
     */
    public function register_scripts_frontend() {

        wp_enqueue_script( 'default_scripts',  $this->Config->scriptsPath . 'seating-frontend.js', array(), $this->Config->PluginData['Version'], true  );
        wp_enqueue_style( 'fooevents-seating-style',  $this->Config->stylesPath . 'seating.css', array(), $this->Config->PluginData['Version']  );
        wp_enqueue_style( 'wp-jquery-ui-dialog' );
        wp_enqueue_script( 'jquery-ui-dialog' );

        
        wp_localize_script('default_scripts', 'localObj', $this->get_translations());
        
    }   
    
    /**
     * Initializes the WooCommerce meta box
     * 
     */
    public function add_product_seating_options_tab() {

        echo '<li class="custom_tab_seating_options"><a href="#fooevents_seating_options">'.__( 'Event Seating', 'fooevents-seating' ).'</a></li>';
        
    }

    /**
     * Add seating tabs 
     * 
     * 
     */
    public function add_product_seating_options_tab_options() {
        
        global $post;
        
        $fooevents_seating_options_serialized = get_post_meta($post->ID,'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

        $fooevents_seats_unavailable_serialized = get_post_meta($post->ID,'fooevents_seats_unavailable_serialized', true);
        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_blocked_serialized = get_post_meta($post->ID,'fooevents_seats_blocked_serialized', true);
        if(empty($fooevents_seats_blocked_serialized)) {
            $fooevents_seats_blocked_serialized = json_encode(array());
        } 
        $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
        $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_aisles_serialized = get_post_meta($post->ID,'fooevents_seats_aisles_serialized', true);
        if(empty($fooevents_seats_aisles_serialized)) {
            $fooevents_seats_aisles_serialized = json_encode(array());
        } 
        $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
        $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);
    
        if(empty($fooevents_seating_options)) {
            
            $fooevents_seating_options = array();
            
        }
        
        require($this->Config->templatePath.'seating_options.php');
        
    }
    
    /**
     * Display seating options 
     * 
     */
    public function get_seating_options() {
        
        ob_start();
        
        $globalWooCommerceEventsSeatingColor = get_option('globalWooCommerceEventsSeatingColor');
        $globalWooCommerceEventsSeatingColorSelected = get_option('globalWooCommerceEventsSeatingColorSelected');
        $globalWooCommerceEventsSeatingColorUnavailableSelected = get_option('globalWooCommerceEventsSeatingColorUnavailableSelected');
        
        require($this->Config->templatePath.'seating-settings.php');
        
        return ob_get_clean();
        
    }
    
    /**
     * Register seating options
     * 
     */
    public function register_seating_options() {
        
        register_setting('fooevents-seating-settings', 'globalWooCommerceEventsSeatingColor');
        register_setting('fooevents-seating-settings', 'globalWooCommerceEventsSeatingColorSelected');
        register_setting('fooevents-seating-settings', 'globalWooCommerceEventsSeatingColorUnavailableSelected');
        
    }
    
    /**
     * Processes the meta box form once the publish / update button is clicked.
     * 
     * @global object $woocommerce_errors
     * @param int $post_id
     */
    public function process_meta_box($post_id) {
        
        global $woocommerce_errors;

        if(isset($_POST['fooevents_seating_options_serialized'])) {
            /* Data that comes from the seating form */

            $fooevents_seating_options_serialized = stripslashes($_POST['fooevents_seating_options_serialized']);
            if(empty($fooevents_seating_options_serialized)) {
                $fooevents_seating_options_serialized = json_encode(array());
            }
            $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
            $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, JSON_UNESCAPED_UNICODE);

            $fooevents_seats_unavailable_serialized = stripslashes($_POST['fooevents_seats_unavailable_serialized']);
            if(empty($fooevents_seats_unavailable_serialized)) {
                $fooevents_seats_unavailable_serialized = json_encode(array());
            } 
            $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
            $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

            $fooevents_seats_blocked_serialized = stripslashes($_POST['fooevents_seats_blocked_serialized']);
            if(empty($fooevents_seats_blocked_serialized)) {
                $fooevents_seats_blocked_serialized = json_encode(array());
            } 
            $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
            $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

            $fooevents_seats_aisles_serialized = stripslashes($_POST['fooevents_seats_aisles_serialized']);
            if(empty($fooevents_seats_aisles_serialized)) {
                $fooevents_seats_aisles_serialized = json_encode(array());
            } 
            $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
            $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);
            
               
            update_post_meta($post_id, 'fooevents_seating_options_serialized', $fooevents_seating_options_serialized);
            update_post_meta($post_id, 'fooevents_seats_unavailable_serialized', $fooevents_seats_unavailable_serialized);
            update_post_meta($post_id, 'fooevents_seats_blocked_serialized', $fooevents_seats_blocked_serialized);
            update_post_meta($post_id, 'fooevents_seats_aisles_serialized', $fooevents_seats_aisles_serialized);

        }
        
    }
    
    /**
     * Register plugin scripts.
     * 
     */
    public function register_scripts() {
        
        wp_enqueue_script('jquery');
        wp_enqueue_script('fooevents-seating-script',  $this->Config->scriptsPath . 'seating.js', array(), $this->Config->PluginData['Version'], true);
        wp_enqueue_script('jquery-ui-dialog');

        if(isset($_GET['post_type']) && $_GET['post_type'] == 'event_magic_tickets') {
            
            wp_enqueue_script('events-seating-admin-add-ticket', $this->Config->scriptsPath . 'events-seating-admin-add-ticket.js', array('jquery'), $this->Config->PluginData['Version'], true);
            wp_localize_script('events-seating-admin-add-ticket', 'FooEventSeatingAddTicketObj', array('ajaxurl' => admin_url('admin-ajax.php')));
            
        }

        wp_localize_script('fooevents-seating-script', 'localObj', $this->get_translations());

    }

    /**
     * Get translatable terms
     * 
     */
    public function get_translations() {
        
        return array(
            'chartAvailable'                => __('Available', 'fooevents-seating'),
            'chartBooked'                   => __('Already Booked', 'fooevents-seating'),
            'chartBlocked'                  => __('Blocked', 'fooevents-seating'),
            'chartDifferentSelected'        => __('Selected for another attendee in this order', 'fooevents-seating'),
            'chartThisSelected'             => __('Selected for this attendee', 'fooevents-seating'),
            'chartFront'                    => __('FRONT', 'fooevents-seating'),
            'chartSelectSeats'              => __('Select seats', 'fooevents-seating'),
            'chartSelectSeat'               => __('Select seat', 'fooevents-seating'),
            'chartNoSeatsToShow'            => __("No seats to show. Add rows and seats by clicking on the '+ New Row' button.", 'fooevents-seating'),
            'chartSelectedForAttendee'      => __('Selected seat', 'fooevents-seating'),
            'chartNoSeatsAvailable'         => __('(No seats available)', 'fooevents-seating'),
            'chartSeat'                     => __('Seat', 'fooevents-seating'),
            'chartAlreadySelected'          => __('is already selected. Please select another seat.', 'fooevents-seating'),
            'selectedSeatsAvailable'        => __('Make selected seats available', 'fooevents-seating'),
            'refreshSeats'                  => __('Refresh seating chart', 'fooevents-seating'),
            'selectedSeatsUnavailable'      => __('Make selected seats unavailable', 'fooevents-seating'),
            'selectedSeatsBlock'            => __('Block selected seats', 'fooevents-seating'),
            'selectedSeatsAddAisles'        => __('Add aisle to the right of selected seats', 'fooevents-seating'),
            'selectedSeatsRemoveAisles'     => __('Remove aisle from selected seats', 'fooevents-seating'),
            'selectedSeatsApply'            => __('Apply', 'fooevents-seating'),
            'selectedSeatsDisclaimer1'      => __("DISCLAIMER: Please note that the action of making seats available or unavailable cannot be reversed. You have to 'Update' your event before changes will take effect. Making seats available will mean that your customers will be able to select these seats again. This could result in more than one attendee booking the same seat. Only use this functionality if you are sure that this will not result in double bookings. Before making any changes, click on the 'Refresh seating chart' button to ensure that you are working with the latest seating chart.", 'fooevents-seating'),
            'selectedSeatsDisclaimer2'      => __("DISCLAIMER: Please note that the action of reassigning seats cannot be reversed. You have to 'Update' this ticket before changes will take effect. Making seats available will mean that your customers will be able to select these seats again. This could result in more than one attendee booking the same seat. Only use this functionality if you are sure that this will not result in double bookings.", 'fooevents-seating'),
            'ajaxurl'                       => admin_url('admin-ajax.php'),
        );
       
    }
    
    /**
     * Register plugin styles.
     * 
     */
    public function register_styles() {
        
        wp_enqueue_style('fooevents-seating-style', $this->Config->stylesPath.'seating.css', array(), $this->Config->PluginData['Version']);
        wp_enqueue_style( 'wp-jquery-ui-dialog' );

    }
    
    /**
     * Checks if FooEvents is installed
     * 
     */
    public function check_fooevents() {
        
        $fooevents_path = WP_PLUGIN_DIR.'/fooevents/fooevents.php';

        $plugin_data = get_plugin_data($fooevents_path);
        
        if (!function_exists('is_plugin_active') || !function_exists('is_plugin_active_for_network')) {

            require_once(ABSPATH.'/wp-admin/includes/plugin.php' );

        }
        
        if (!is_plugin_active('fooevents/fooevents.php')) {

            $this->output_notices(array(__('The FooEvents Seating plugin requires FooEvents for WooCommerce to be installed.', 'fooevents-seating')));

        } 
    }

    /**
     * Selects correct new variations for each seating row when product is duplicated
     * 
     * 
     */
    public function duplicate_seat_variation_options($duplicate, $product) {

        $x = 0;
        $newVarIDs = $duplicate->get_children();

        $fooevents_seating_options = $this->correct_legacy_options(json_decode(get_post_meta($duplicate->get_id(),'fooevents_seating_options_serialized', true), true));
        foreach($product->get_children() as $origVarID) {

            /* Loop through each new duplicated seating options array to match variation IDs */
            foreach(array_keys($fooevents_seating_options) as $seatOptionKey) {
                $thisSeatVarID = $fooevents_seating_options[$seatOptionKey][$seatOptionKey . "_variations"];
                if ($origVarID == $thisSeatVarID) {
                    $fooevents_seating_options[$seatOptionKey][$seatOptionKey . "_variations"] = $newVarIDs[$x];
                }
            }

            $x++;
        }
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options);
        
        $duplicate->update_meta_data('fooevents_seating_options_serialized', $fooevents_seating_options_serialized);
        $duplicate->save_meta_data();

    }
    
    
    /**
     * Outputs the seating chart on the checkout screen
     * 
     * @param int $product_ID
     * @param int $x
     * @param int $y
     * @param array $ticket
     * @param object $checkout
     */
    public function output_seating_fields($product_ID, $x, $y, $ticket, $checkout, $tickets) {


        $WooCommerceEventsViewSeatingChart = get_post_meta($product_ID, 'WooCommerceEventsViewSeatingChart', true);

        echo '<script type="text/javascript"> if (!fooevents_seats_options_serialized) { var fooevents_seats_options_serialized = []; } </script>';
        echo '<script type="text/javascript"> if (!fooevents_seats_unavailable_serialized) { var fooevents_seats_unavailable_serialized = []; } </script>';
        echo '<script type="text/javascript"> if (!fooevents_seats_blocked_serialized) { var fooevents_seats_blocked_serialized = []; } </script>';
        echo '<script type="text/javascript"> if (!fooevents_seats_aisles_serialized) { var fooevents_seats_aisles_serialized = []; } </script>';
        echo '<script type="text/javascript"> var seatColor = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColor', true)) . '";</script>';
        echo '<script type="text/javascript"> var seatColorSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorSelected', true)) . '";</script>';
        echo '<script type="text/javascript"> var seatColorUnavailableSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorUnavailableSelected', true)) . '";</script>';

        echo '<script type="text/javascript"> if (!selectedSeat) { var selectedSeat = {}; }</script>';

        global $woocommerce;
        
        $required = true;
        

        if (!empty($ticket['variation_id'])) {
            
            $varID = $ticket['variation_id'];
            
        } else {

            $varID = 0;

        }
        
        $any_variations = "";

        $fooevents_seating_data = array();
        $current_selected_var = array();
        $selected_seating_var_option = "";

        $fooevents_seating_options_serialized = get_post_meta($product_ID,'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

        $fooevents_seats_unavailable_serialized = get_post_meta($product_ID,'fooevents_seats_unavailable_serialized', true);
        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_blocked_serialized = get_post_meta($product_ID,'fooevents_seats_blocked_serialized', true);
        if(empty($fooevents_seats_blocked_serialized)) {
            $fooevents_seats_blocked_serialized = json_encode(array());
        } 
        $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
        $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_aisles_serialized = get_post_meta($product_ID,'fooevents_seats_aisles_serialized', true);
        if(empty($fooevents_seats_aisles_serialized)) {
            $fooevents_seats_aisles_serialized = json_encode(array());
        } 
        $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
        $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);
        
        if (!empty($fooevents_seating_options)) {
            
            foreach($fooevents_seating_options as $row_option_key => $row_object) {
                
                $row_key = str_replace('_option', '', $row_option_key);
                
                $fooevents_seating_data[$row_key . '_row_name'] = array(
                    'number_seats' => $row_object[$row_key . '_number_seats'],
                    'variation' => $row_object[$row_key . '_variations']
                );
                
            }

            echo '<script type="text/javascript">fooevents_seating_data["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seating_data) . '\');</script>';
            echo '<script type="text/javascript">fooevents_seats_unavailable["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_unavailable) .'\');</script>';
            echo '<script type="text/javascript">fooevents_seats_blocked["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_blocked) .'\');</script>';
            echo '<script type="text/javascript">fooevents_seats_aisles["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_aisles) .'\');</script>';
            
            if (!empty($fooevents_seats_unavailable_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_unavailable_serialized['. $product_ID . ']  = ' . $fooevents_seats_unavailable_serialized . '</script>';
            }

            if (!empty($fooevents_seats_blocked_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_blocked_serialized['. $product_ID . ']  = ' . $fooevents_seats_blocked_serialized . '</script>';
            }

            if (!empty($fooevents_seats_aisles_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_aisles_serialized['. $product_ID . ']  = ' . $fooevents_seats_aisles_serialized . '</script>';
            }
            
            echo '<script type="text/javascript"> fooevents_seats_options_serialized[' . $product_ID . '] = ' . $fooevents_seating_options_serialized . '</script>';

            /* The general event variation for the current attendee */
            $selected_seating_var_option = "";
            if (!empty($ticket['variation_id'])) {
                $selected_seating_var_option = new WC_Product_Variation($ticket['variation_id']);
                if (get_post_meta($ticket['variation_id'] , 'fooevents_variation_seating_required', true ) == "no")
                    $required = false;
                else
                    $required = true;
                $selected_seating_var_option = $selected_seating_var_option->get_attributes();

                $selected_seating_var_option_keys = array_keys($selected_seating_var_option);
                foreach($selected_seating_var_option_keys as $key => $option) {

                    array_push($current_selected_var, $selected_seating_var_option[$option]);
            
                }
                
            }



            if(!empty($fooevents_seating_options) && $fooevents_seating_options != "{}") {
                $select_values_row = array("" => __("Select row", 'fooevents-seating'));
                $completely_different = 0;
                $has_any_variations = 0;

                $show_seats = false;

                foreach($fooevents_seating_options as $option_key => $option) {
                    
                    $option_ids = array_keys($option);
                    $option_values = array_values($option);
                    $option_label_output = __('Seat', 'fooevents-seating');
                    $selectLabel = $option_label_output;

                    $select_values_row[$option_ids[0]] = $option_values[0];
                     
                    /* This is the variation selected for this row in the seating tab */
                    $this_var_option = new WC_Product_Variation($option_values[2]);

                    if ($option_values[2] == "default") {
                        $show_seats = true;
                    }
                 
               
                    foreach($this_var_option->get_attributes() as $key_var => $var_attr) {

                        if (!empty($selected_seating_var_option) && ($selected_seating_var_option[$key_var] != $var_attr) && !empty($var_attr)) {
                            $completely_different = 1;
                        }

                        if (!$completely_different && empty($var_attr)) {
                            $has_any_variations = 1;
                        }

                        if (in_array($var_attr, $current_selected_var)) {
                            $show_seats = true;
                        }
                     
                    }

                    /* Make variable that says which variations should show */
                   if (!($completely_different) && $has_any_variations) {
                       
                        if (empty($any_variations)) {

                            $any_variations = $option_values[2];
                            
                        }
                        
                    }
           
                    $completely_different = 0;
                    $has_any_variations = 0;

                }

                
                $selected_seat_row = "";
                $selected_seat_nr_array = array();
                $selected_seat_nr = array();


                if (!empty($ticket['seats'])) {
                    
                    foreach($tickets as $ticket) {
                        $selected_seat_nr_array = explode(",", $ticket['seats']);
                        foreach($selected_seat_nr_array as $seat) {
                            if (!in_array($seat, $selected_seat_nr)) {
                                array_push($selected_seat_nr, $seat);

                            }
                        }
                    }              

                    $selected_seat_row = substr($selected_seat_nr[$y-1], 0, strpos($selected_seat_nr[$y-1],"_")) . "_row_name";
                    array_unshift($selected_seat_nr,"");
                    echo '<script type="text/javascript"> selectedSeat[' . $x . '] = JSON.parse(\'' . json_encode($selected_seat_nr) . '\');</script>';

                }

                if ($show_seats) {
         
                    $selectParamsRow = array(
                        'type'          => 'select',
                        'class'         => array("seating-class seating-class-row form-row-wide fooevents_event_$product_ID fooevents_variation_$varID"),
                        'label'         => $selectLabel,
                        'placeholder'   => 'Row Name',
                        'options'       => $select_values_row,
                        'default'       => $selected_seat_row,
                        'custom_attributes' => array("data-value" => $selected_seat_row),
                        'required'      => $required,   
                    );
                    
                    $selectParamsSeat = array(
                        'type'          => 'select',
                        'class'         => array("seating-class seating-class-seat form-row-wide fooevents_event_$product_ID fooevents_variation_$varID"),
                        'label'         => '',
                        'placeholder'   => 'Seat Number',
                        'options'       => array(__("Select seat", 'fooevents-seating')),
                        'required'      => $required,    
                    );

                    woocommerce_form_field("fooevents_seat_row_name_".$x."__".$y, $selectParamsRow, $checkout->get_value("fooevents_seat_row_name_".$x.'__'.$y ));
                    woocommerce_form_field("fooevents_seat_number_".$x."__".$y, $selectParamsSeat, $checkout->get_value("fooevents_seat_number_".$x.'__'.$y ));

                    if ($WooCommerceEventsViewSeatingChart == "on") {
                        echo "<a href='javascript:void(0)' name='fooevents_variation_" . $varID . "' class='fooevents_seating_chart button button-primary' id='fooevents_event_id_" . $product_ID . "'>";
                        echo _e("View seating chart", 'fooevents-seating');
                        echo "</a> <div id='fooevents_seating_dialog' title='" . __("Seating Chart", 'fooevents-seating') . "'></div>";
                    }
                    if (!empty($ticket['seat_number'])) {

                        echo '<script type="text/javascript"> selectedSeat = "' . $ticket['seat_number'] . '";</script>';
                        
                    }
 
                    
                }
  
            }
                
        }

       echo '<script type="text/javascript"> anyVariations[' . $varID . '] = "' . $any_variations . '";</script>';

    }


     /**
     * Get the row name
     * 
     */
    public function get_row_name($eventID, $code){

        $fooevents_seating_options_serialized = get_post_meta($eventID,'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = json_decode($fooevents_seating_options_serialized, true);
        
        if (!empty($fooevents_seating_options)) {
            
            foreach($fooevents_seating_options as $row_option_key => $row_object) {

                if (strpos($row_option_key, $code) === 0) {

                    return $row_object[$code . "_row_name"];
                }

            }
        }

    }

    /**
     * Saves tickets meta box settings
     * 
     * @param int $post_ID
     * @global object $post
     * @global object $woocommerce
     * 
     */
    public function save_ticket_meta_boxes($post_ID) {
        
        global $post;
        global $woocommerce;

        $this->remove_unavailable_seats_from_event($post_ID);
       
        if (!empty($_POST["meta"])) {
            foreach($_POST["meta"] as $field => $array_value) {

                if (strpos($array_value["key"], 'fooevents_seat_row_name_') === 0) {

                    delete_post_meta($post_ID, $array_value["key"]);
                }
                if (strpos($array_value["key"], 'fooevents_seat_number_') === 0) {
                
                    delete_post_meta($post_ID, $array_value["key"]);
                }

            }
        }

        $eventID = get_post_meta($post_ID, 'WooCommerceEventsProductID', true);
        if (empty($eventID) && isset($_POST["WooCommerceEventsEvent"])) {
            $eventID = $_POST["WooCommerceEventsEvent"];
        }
            
        if (is_object($post) && isset($_POST)) {

            if($post->post_type == "event_magic_tickets") {
                
                foreach($_POST as $key => $value) {


                    if ($key == 'fooevents_seat_row_name') {

                        $row_value = sanitize_text_field($value);
                        $row_value_var_code = substr($row_value, 0, strpos($row_value, "_"));
                        $row_name = $this->get_row_name($eventID, $row_value_var_code);
                        update_post_meta($post_ID, 'fooevents_seat_row_name_' . $row_value_var_code, $row_name);

                    }
                

                    if (strpos($key, 'fooevents_seat_number') === 0) {

                        $seat_value = substr($value, strrpos( $value, '_' ) + 1 );
                        $seat_value = sanitize_text_field($seat_value);
                        update_post_meta($post_ID, 'fooevents_seat_number_' . $row_value_var_code, $seat_value);

                    }
                    
                }
                
            }
            
        }

        $this->add_unavailable_seat_to_event($post_ID);

    }

    
    /**
     * Checks a final time to see if the selected seating fields on checkout screen are available when the Checkout button is clicked.
     * 
     * @param int $event_id
     * @param int $event
     * @param int $x
     * @param int $y
     */
    public function check_required_field_availability($event_id, $event, $x, $y) {
        $fooevents_seats_unavailable_serialized = get_post_meta($event_id, 'fooevents_seats_unavailable_serialized', true);
        $fooevents_seats_blocked_serialized = get_post_meta($event_id, 'fooevents_seats_blocked_serialized', true);
        $fooevents_seats_aisles_serialized = get_post_meta($event_id, 'fooevents_seats_aisles_serialized', true);

        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = json_decode($fooevents_seats_unavailable_serialized, true);

        if(empty($fooevents_seats_blocked_serialized)) {
            $fooevents_seats_blocked_serialized = json_encode(array());
        } 
        $fooevents_seats_blocked = json_decode($fooevents_seats_blocked_serialized, true);

        if(empty($fooevents_seats_aisles_serialized)) {
            $fooevents_seats_aisles_serialized = json_encode(array());
        } 
        $fooevents_seats_aisles = json_decode($fooevents_seats_aisles_serialized, true);
        
        $field_id = "fooevents_seat_number".'_'.$x.'__'.$y;

        if (!empty($fooevents_seats_unavailable[0]) && !empty($_POST[$field_id])) {
            if (($key = array_search($_POST[$field_id], $fooevents_seats_unavailable)) !== false) {
                $notice = sprintf(__( 'The seat for %s attendee %d has already been booked. Please select a different seat.', 'fooevents-seating' ), $event, $y );
                wc_add_notice( $notice, 'error' );
            }  

        }
        
        if (!empty($fooevents_seats_blocked[0]) && !empty($_POST[$field_id])) {
            if (($key = array_search($_POST[$field_id], $fooevents_seats_blocked)) !== false) {
                $notice = sprintf(__( 'The seat for %s attendee %d has been blocked and is not available for booking. Please select a different seat.', 'fooevents-seating' ), $event, $y );
                wc_add_notice( $notice, 'error' );
            }  

        }

    }

    /**
     * Checks the seating fields on checkout screen.
     * 
     * @param array $ticket
     * @param int $event
     * @param int $x
     * @param int $y
     */
    public function check_required_fields($ticket, $event, $x, $y) {
        
        global $woocommerce;
        $seat_selected = false;
        
        $fooevents_seating_options_serialized = get_post_meta($ticket['product_id'], 'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

        $fooevents_seats_unavailable_serialized = get_post_meta($ticket['product_id'],'fooevents_seats_unavailable_serialized', true);
        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_blocked_serialized = get_post_meta($ticket['product_id'],'fooevents_seats_blocked_serialized', true);
        if(empty($fooevents_seats_blocked_serialized)) {
            $fooevents_seats_blocked_serialized = json_encode(array());
        } 
        $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
        $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_aisles_serialized = get_post_meta($ticket['product_id'],'fooevents_seats_aisles_serialized', true);
        if(empty($fooevents_seats_aisles_serialized)) {
            $fooevents_seats_aisles_serialized = json_encode(array());
        } 
        $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
        $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);

        if(!empty($fooevents_seating_options)) {
           
            foreach($fooevents_seating_options as $option_key => $option) {
                
                $option_ids = array_keys($option);
                $option_values = array_values($option);
                $field_id = "fooevents_seat_number".'_'.$x.'__'.$y;
                
                if (!empty($_POST[$field_id])) {
                    $seat_selected = true;

                }   
            }
  
            if (!$seat_selected && (get_post_meta($ticket['variation_id'] , 'fooevents_variation_seating_required', true ) == "yes" || empty(get_post_meta($ticket['variation_id'] , 'fooevents_variation_seating_required', true )))) {
                $notice = sprintf(__( 'A row and seat number is required for %s attendee %d', 'fooevents-seating' ), $event, $y );
                wc_add_notice( $notice, 'error' );
            }
        }

    }
 
    /**
     * Displays seating details on the tickets detail page.
     * 
     * @param int $post
     */
    public function display_tickets_meta_seat_options($post) {
        
        $post_meta = get_post_meta($post->ID); 
        $custom_values = array();
        
        foreach($post_meta as $key => $meta) {
            
           if (strpos($key, 'fooevents_seat_') === 0) {
               
                $custom_values[$key] = ucwords(str_replace('_', " ", $meta[0])); 
       
           }
            
        }
        
        if(!empty($custom_values)) {

            require($this->Config->templatePath.'seating-tickets.php');
            
        }

    }

    /**
     * Display seats on tickets.
     * 
     * @param int $ID
     * @return array
     */
    public function display_tickets_meta_seat_options_output($ID) {
        
        $post_meta = get_post_meta($ID); 
        $seat_values = array();
        

        foreach($post_meta as $key => $meta) {
            
           if (strpos($key, 'fooevents_seat_') === 0) {
               
                $custom_values[$key] = $meta[0];
                $meta[0] = str_replace('_', " ", $meta[0]);
                $meta[0] = ucwords($meta[0]);    
                
                if (strpos($key, 'fooevents_seat_row_name') === 0) {

                    $seat_values['row_name_label'] = __( 'Row Name', 'fooevents-seating' );
                    $seat_values['row_name'] = $meta[0];

                }

                if (strpos($key, 'fooevents_seat_number') === 0) {

                    $seat_values['seat_number_label'] = __( 'Seat Number', 'fooevents-seating' );
                    $seat_values['seat_number'] = $meta[0];

                }  
                
           }
            
        }

        return $seat_values;
        
    }


     /**
     * Display seats on tickets legacy.
     * 
     * @param int $ID
     * @return string
     */
    //LEGACY: 20201028
    public function display_tickets_meta_seat_options_output_legacy($ID) {
        
        $post_meta = get_post_meta($ID); 
        $output = '';
        
        foreach($post_meta as $key => $meta) {
            
           if (strpos($key, 'fooevents_seat_') === 0) {
               
                $custom_values[$key] = $meta[0];
                $meta[0] = str_replace('_', " ", $meta[0]);
                $meta[0] = ucwords($meta[0]);    
                
                $output .= $this->_output_seating_field_name($key).': '.$meta[0]."<br>";
                
           }
            
        }
        
        return $output;
        
    }
    //ENDLEGACY: 20201028
    
    /**
     * Formats custom attendee fields for CSV
     * 
     * @param int $ID
     * @return array
     */
    public function display_tickets_meta_seat_options_array($ID) {
        
        $post_meta = get_post_meta($ID); 
        $custom_values = array();
        
        foreach($post_meta as $key => $meta) {
            
            if (strpos($key, 'fooevents_seat_') === 0) {
               
                if ($key != 'fooevents_seat_row_name' && $key != 'fooevents_seat_number') {

                    $key = substr($key, 0, strrpos($key, "_"));

                }

                $key = substr($key, 10);
                $key = str_replace('_', ' ', $key);
                $option_label = ucwords($key);
                $custom_values[$option_label] = esc_attr($meta[0]);
                
           }
            
        }
 
        return $custom_values;
        
    }
    
    /**
     * Captures seating options on checkout.
     * 
     * @param int $postID
     * @param int $product_id
     * @param int $x
     */
    
    public function capture_seating_options($product_ID, $x, $y) {
 
        $custom_values = array();

        $fooevents_seating_options_serialized = get_post_meta($product_ID,'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

        $fooevents_seats_unavailable_serialized = get_post_meta($product_ID,'fooevents_seats_unavailable_serialized', true);
        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

        $seatTaken= "";
        foreach($_POST as $key => $value) {

            if (strpos($key, 'fooevents_seat_row_name') === 0) {

                if ($key == 'fooevents_seat_row_name_' . $x.'__'.$y) {

                    $id_value = substr($value, 0, strpos($value, "_row_name"));
                    $field_name = preg_replace('/'. preg_quote('_'.$x.'__'.$y, '/') . '$/', '', $key) . "_" . $id_value;
                    $custom_values[$field_name] = $fooevents_seating_options[$id_value][$id_value . "_row_name"];
                    $seatTaken = $id_value . "_number_seats_";
                    
                }
                
            }

            if (strpos($key, 'fooevents_seat_number') === 0) {

                if ($key == 'fooevents_seat_number_' . $x.'__'.$y) {

                    $field_name = preg_replace('/'. preg_quote('_'.$x.'__'.$y, '/') . '$/', '', $key) . "_" . $id_value;
                    $seat_value = substr($value, strrpos( $value, '_' ) + 1 );
                    $seatTaken .= $seat_value;
                    $custom_values[$field_name] = $seat_value;
                    
                }
                
            }
           
        }
        
        array_push($fooevents_seats_unavailable, $seatTaken);
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);
        update_post_meta($product_ID, "fooevents_seats_unavailable_serialized", $fooevents_seats_unavailable_serialized);
        return $custom_values;
        
    }
    
    /**
     * Capture seating options 
     * 
     * @param int $postID
     * @param array $WooCommerceEventsSeatingFields
     */
    public function process_capture_seating_options($postID, $WooCommerceEventsSeatingFields) {
        
        foreach($WooCommerceEventsSeatingFields as $key => $value) {
         
            if (strpos($key, 'fooevents_seat_') === 0) {
                
                update_post_meta($postID, $key, $value);
                
            }
            
        }
        
        update_post_meta($postID, "WooCommerceEventsSeatingFields", $WooCommerceEventsSeatingFields);
        
    }


    /**
     * Outputs seating options in admin
     * 
     * @param int $ticket_post_ID
     * @param int $product_ID
     * @return string
     * 
     */
    public function ticket_details_seating_fields($product_ID, $ticket_post_ID) {
        
        $seat_options = "";

        $fooevents_seating_options_serialized = get_post_meta($product_ID,'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

        $fooevents_seats_unavailable_serialized = get_post_meta($product_ID,'fooevents_seats_unavailable_serialized', true);
        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_blocked_serialized = get_post_meta($product_ID,'fooevents_seats_blocked_serialized', true);
        if(empty($fooevents_seats_blocked_serialized)) {
            $fooevents_seats_blocked_serialized = json_encode(array());
        } 
        $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
        $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_aisles_serialized = get_post_meta($product_ID,'fooevents_seats_aisles_serialized', true);
        if(empty($fooevents_seats_aisles_serialized)) {
            $fooevents_seats_aisles_serialized = json_encode(array());
        } 
        $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
        $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);

        echo '<script type="text/javascript">var seatColor = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColor', true)) . '";</script>';
        echo '<script type="text/javascript">var seatColorSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorSelected', true)) . '";</script>';
        echo '<script type="text/javascript">var seatColorUnavailableSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorUnavailableSelected', true)) . '";</script>';
        
        echo '<script type="text/javascript">var fooevents_seating_options_serialized = JSON.parse(\'' . json_encode($fooevents_seating_options) .'\');</script>';
        echo '<script type="text/javascript">var fooevents_seats_options_serialized = new Object(); fooevents_seats_options_serialized[' . $product_ID . '] = ' . $fooevents_seating_options_serialized . '</script>';
        echo '<script type="text/javascript">var fooevents_seats_unavailable_serialized = JSON.parse(\'' . json_encode($fooevents_seats_unavailable) .'\');</script>';
        echo '<script type="text/javascript">var fooevents_seats_blocked_serialized = JSON.parse(\'' . json_encode($fooevents_seats_blocked) .'\');</script>';
        echo '<script type="text/javascript">var fooevents_seats_aisles_serialized = JSON.parse(\'' . json_encode($fooevents_seats_aisles) .'\');</script>';
        
        $post_meta = get_post_meta($ticket_post_ID);
        $seat_row_values = array();
        $seat_number_values = array();
        $selected_row = "";
        $fooevents_seating_data = array();
        $selected_field_id = 0;

        /* GET seat variable code */
        foreach($post_meta as $key => $meta) {

           if (strpos($key, 'fooevents_seat_row_name') === 0) {
                
                $field = explode('_', $key);

                if (isset($field[4])) {

                    $selected_field_id = $field[4];
                    
                } else {

                    $selected_field_id = $meta[0];

                }
                
           }
            
        }


        /* SEE WHICH VARIATION OPTIONS SHOULD SHOW */
        
        $varID = get_post_meta($ticket_post_ID,'WooCommerceEventsVariationID', true);
        
        if (empty($varID) || $varID == "") {

            $varID = 0;

        }
            
        echo '<script type="text/javascript">var event_id = ' . $product_ID . ';</script>';
        echo '<script type="text/javascript">var var_id = ' . $varID . ';</script>';

        /* The general event variation for the current attendee */
        $current_selected_var = array();
        $selected_seating_var_option = "";
        if (!empty($varID)) {
            $selected_seating_var_option = new WC_Product_Variation($varID);
            if (get_post_meta($varID , 'fooevents_variation_seating_required', true ) == "no")
                $required = false;
            else
                $required = true;
            $selected_seating_var_option = $selected_seating_var_option->get_attributes();

            $selected_seating_var_option_keys = array_keys($selected_seating_var_option);
            foreach($selected_seating_var_option_keys as $key => $option) {

                array_push($current_selected_var, $selected_seating_var_option[$option]);
        
            }
            
        }

        
        if (!empty($fooevents_seating_options) && $fooevents_seating_options != "{}") {

            foreach($fooevents_seating_options as $row_option_key => $row_object) {

                $option_values = array_values($row_object);
                $keys = array_keys($row_object);

                /* This is the variation selected for this row in the seating tab */
                $this_var_option = new WC_Product_Variation($option_values[2]);


                $this_var_attr = $this_var_option->get_attributes();
                $this_var_attr_keys = array_keys($this_var_attr);

                if (empty($this_var_attr_keys) && $row_object[$keys[2]] != "default") {

                    continue;

                }
                
                if ($row_object[$keys[2]] == "default" ||
                    (
                        ( count($current_selected_var) == 2 && 
                            ($this_var_attr[$this_var_attr_keys[0]] == $current_selected_var[0] || $this_var_attr[$this_var_attr_keys[0]] == "") &&
                            ($this_var_attr[$this_var_attr_keys[1]] == $current_selected_var[1] || $this_var_attr[$this_var_attr_keys[1]] == "")
                        ) ||
                        ( count($current_selected_var) == 1 && 
                            ($this_var_attr[$this_var_attr_keys[0]] == $current_selected_var[0] || $this_var_attr[$this_var_attr_keys[0]] == "")
                        )

                )) {
                    
                    if (!empty($selected_field_id) && ((strpos($keys[0], $selected_field_id . "_") === 0) || (strpos($keys[0], strtolower($selected_field_id . "_")) === 0))) {

                        $selected_row = "selected";

                    } else {

                        $selected_row = "";

                    }

                    array_push($seat_row_values,array($keys[0], $row_object[$keys[0]], $selected_row));
                
                 }

            }


            foreach($fooevents_seating_options as $row_option_key => $row_object) {
                
                $row_key = str_replace('_option', '', $row_option_key);
                
                $fooevents_seating_data[$row_key . '_row_name'] = array(
                    'number_seats' => $row_object[$row_key . '_number_seats'],
                    'variation' => $row_object[$row_key . '_variations']
                );
                
            }

            if (!empty($fooevents_seats_unavailable_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_unavailable_serialized['. $product_ID . ']  = ' . $fooevents_seats_unavailable_serialized . '</script>';
            }

            if (!empty($fooevents_seats_blocked_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_blocked_serialized['. $product_ID . ']  = ' . $fooevents_seats_blocked_serialized . '</script>';
            }

            if (!empty($fooevents_seats_aisles_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_aisles_serialized['. $product_ID . ']  = ' . $fooevents_seats_aisles_serialized . '</script>';
            }

            $selected_seat = get_post_meta($ticket_post_ID,'fooevents_seat_number_' . $selected_field_id, true);

            if (empty($selected_seat)) {

                if (!empty(get_post_meta($ticket_post_ID,'fooevents_seat_number', true))) {
                    
                    //LEGACY: 20201028
                    $selected_seat = get_post_meta($ticket_post_ID,'fooevents_seat_number', true);
                    //ENDLEGACY: 20201028
    
                } else {
    
                    $selected_seat = 0;
    
                }

            }
            

            echo '<script type="text/javascript">var fooevents_seating_data = new Object(); fooevents_seating_data["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seating_data) . '\');</script>';
            echo '<script type="text/javascript">var fooevents_seats_unavailable = new Object(); fooevents_seats_unavailable["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_unavailable) .'\');</script>';
            echo '<script type="text/javascript">var fooevents_seats_blocked = new Object(); fooevents_seats_blocked["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_blocked) .'\');</script>';
            echo '<script type="text/javascript">var fooevents_seats_aisles = new Object(); fooevents_seats_aisles["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_aisles) .'\');</script>';
            echo '<script type="text/javascript">var fooevents_selected_seat = 0; fooevents_selected_seat = ' . $selected_seat . ';</script>';
            echo '<script type="text/javascript">var fooevents_selected_row = ""; fooevents_selected_row = "' . $selected_field_id . '_row_name";</script>';

        
            ob_start();
        
            require($this->Config->templatePath.'seating_ticket_detail.php'); 
            
            echo "<a href='javascript:void(0)' name='fooevents_variation_" . $varID . "' class='fooevents_seating_chart fooevents_seating_chart_admin button button-primary' id='fooevents_event_id_" . $product_ID . "'>";
            echo _e("View seating chart", 'fooevents-seating');
            echo "</a>";

            $seat_options = ob_get_clean();
        
        }

        
        
        return $seat_options;
        
    }


    /**
     * Loads text-domain for localization
     * 
     */
    public function load_text_domain(){

        $path = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
        $loaded = load_plugin_textdomain( 'fooevents-seating', false, $path);
        
    }
    
    /**
     * Formats field name
     * 
     * @param string $field_name
     * @return string
     */
    private function _output_seating_field_name($field_name) {
        
        if (strpos($field_name, 'fooevents_seat_number') !== false) {
            
            $field_name = str_replace('fooevents_', "", $field_name);
            
        }else {
            
            $field_name = str_replace('fooevents_seat_', "", $field_name);
            
        }
        
        $field_name = substr($field_name, 0, strrpos( $field_name, '_' ));
        $field_name = str_replace('_', " ", $field_name);
        $field_name = ucwords($field_name);
        
        return $field_name;
        
    }
    
    /**
     * Formats field name
     * 
     * @param string $field_name
     * @return string
     */
    private function _encode_custom_field_name($field_name) {
        
        $field_name = str_replace(' ', '_', $field_name);
        $field_name = strtolower($field_name);
        
        return $field_name;
        
    }
    
    /**
     * Outputs notices to screen.
     * 
     * @param array $notices
     */
    private function output_notices($notices) {

        foreach ($notices as $notice) {

                echo "<div class='updated'><p>$notice</p></div>";

        }

    }



    /**
     * Fetch seating options for add ticket page
     * 
     */
    public function fetch_add_ticket_seating_options() {

        $varID = 0;
        $product_ID = sanitize_text_field($_POST['event_id']);
        $variation = wc_get_product($product_ID);

        if ($variation->get_parent_id() != 0) {
            $varID = $product_ID;
            $product_ID = $variation->get_parent_id();
        }
        
        $seat_options = "";

        $fooevents_seating_options_serialized = get_post_meta($product_ID,'fooevents_seating_options_serialized', true);
        $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
        $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

        $fooevents_seats_unavailable_serialized = get_post_meta($product_ID,'fooevents_seats_unavailable_serialized', true);
        if(empty($fooevents_seats_unavailable_serialized)) {
            $fooevents_seats_unavailable_serialized = json_encode(array());
        } 
        $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_unavailable_serialized, true));
        $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_blocked_serialized = get_post_meta($product_ID,'fooevents_seats_blocked_serialized', true);
        if(empty($fooevents_seats_blocked_serialized)) {
            $fooevents_seats_blocked_serialized = json_encode(array());
        } 
        $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
        $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

        $fooevents_seats_aisles_serialized = get_post_meta($product_ID,'fooevents_seats_aisles_serialized', true);
        if(empty($fooevents_seats_aisles_serialized)) {
            $fooevents_seats_aisles_serialized = json_encode(array());
        } 
        $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
        $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);

      
        echo '<script type="text/javascript">var seatColor = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColor', true)) . '";</script>';
        echo '<script type="text/javascript">var seatColorSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorSelected', true)) . '";</script>';
        echo '<script type="text/javascript">var seatColorUnavailableSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorUnavailableSelected', true)) . '";</script>';
        
        echo '<script type="text/javascript">var fooevents_seating_options_serialized = JSON.parse(\'' . json_encode($fooevents_seating_options) .'\');</script>';
        echo '<script type="text/javascript">var fooevents_seats_options_serialized = new Object(); fooevents_seats_options_serialized[' . $product_ID . '] = ' . $fooevents_seating_options_serialized . '</script>';
        echo '<script type="text/javascript">var fooevents_seats_unavailable_serialized = JSON.parse(\'' . json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE) .'\');</script>';
        echo '<script type="text/javascript">var fooevents_seats_blocked_serialized = JSON.parse(\'' . json_encode($fooevents_seats_blocked) .'\');</script>';
        echo '<script type="text/javascript">var fooevents_seats_aisles_serialized = JSON.parse(\'' . json_encode($fooevents_seats_aisles) .'\');</script>';
        
        $seat_row_values = array();
        $seat_number_values = array();
        $selected_row = "";
        $fooevents_seating_data = array();
        $selected_field_id = 0;

        echo '<script type="text/javascript">var event_id = ' . $product_ID . ';</script>';
        echo '<script type="text/javascript">var var_id = ' . $varID . ';</script>';

        $current_selected_var = array();
        /* The general event variation for the current attendee */
        $selected_seating_var_option = "";
        if (!empty($varID)) {
            $selected_seating_var_option = new WC_Product_Variation($varID);
            if (get_post_meta($varID , 'fooevents_variation_seating_required', true ) == "no")
                $required = false;
            else
                $required = true;
            $selected_seating_var_option = $selected_seating_var_option->get_attributes();

            $selected_seating_var_option_keys = array_keys($selected_seating_var_option);
            foreach($selected_seating_var_option_keys as $key => $option) {

                array_push($current_selected_var, $selected_seating_var_option[$option]);
        
            }
            
        }

        if (!empty($fooevents_seating_options) && $fooevents_seating_options != "{}") {
            
            foreach($fooevents_seating_options as $row_option_key => $row_object) {


                $option_values = array_values($row_object);
                $keys = array_keys($row_object);

                /* This is the variation selected for this row in the seating tab */
                $this_var_option = new WC_Product_Variation($option_values[2]);

                $this_var_attr = $this_var_option->get_attributes();
                $this_var_attr_keys = array_keys($this_var_attr);

                if (empty($this_var_attr_keys) && $row_object[$keys[2]] != "default") {

                    continue;

                }
                
                if ($row_object[$keys[2]] == "default" ||
                (( count($current_selected_var) == 2 && 
                        ($this_var_attr[$this_var_attr_keys[0]] == $current_selected_var[0] || $this_var_attr[$this_var_attr_keys[0]] == "") &&
                        ($this_var_attr[$this_var_attr_keys[1]] == $current_selected_var[1] || $this_var_attr[$this_var_attr_keys[1]] == "")
                    ) ||
                    ( count($current_selected_var) == 1 && 
                        ($this_var_attr[$this_var_attr_keys[0]] == $current_selected_var[0] || $this_var_attr[$this_var_attr_keys[0]] == "")
                    ))) {
                    
                    array_push($seat_row_values,array($keys[0], $row_object[$keys[0]], $selected_row));
                
                }

            }


            foreach($fooevents_seating_options as $row_option_key => $row_object) {
        
                $row_key = str_replace('_option', '', $row_option_key);
                $fooevents_seating_data[$row_key . '_row_name'] = array(
                    'number_seats' => $row_object[$row_key . '_number_seats'],
                    'variation' => $row_object[$row_key . '_variations']
                );
                
            }

            if (!empty($fooevents_seats_unavailable_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_unavailable_serialized['. $product_ID . ']  = ' . $fooevents_seats_unavailable_serialized . '</script>';
            }

            if (!empty($fooevents_seats_blocked_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_blocked_serialized['. $product_ID . ']  = ' . $fooevents_seats_blocked_serialized . '</script>';
            }

            if (!empty($fooevents_seats_aisles_serialized)) {
                echo '<script type="text/javascript">fooevents_seats_aisles_serialized['. $product_ID . ']  = ' . $fooevents_seats_aisles_serialized . '</script>';
            }

            echo '<script type="text/javascript">var fooevents_seating_data = new Object(); fooevents_seating_data["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seating_data) . '\');</script>';
            echo '<script type="text/javascript">var fooevents_seats_unavailable = new Object(); fooevents_seats_unavailable["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE) .'\');</script>';
            echo '<script type="text/javascript">var fooevents_seats_blocked = new Object(); fooevents_seats_blocked["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_blocked) .'\');</script>';
            echo '<script type="text/javascript">var fooevents_seats_aisles = new Object(); fooevents_seats_aisles["fooevents_event_' . $product_ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_aisles) .'\');</script>';
            echo '<script type="text/javascript">var fooevents_selected_seat = 0; </script>';
            echo '<script type="text/javascript">var fooevents_selected_row = ""; fooevents_selected_row = "' . $selected_field_id . '_row_name";</script>';

            require_once($this->Config->templatePath.'seating_add_ticket.php');

            echo "<a href='javascript:void(0)' name='fooevents_variation_" . $varID . "' class='fooevents_seating_chart_admin button button-primary' id='fooevents_event_id_" . $product_ID . "'>";
            echo _e("View seating chart", 'fooevents-seating');
            echo "</a>";

        }  
     
        exit();
        
    }



    function refresh_seating_chart() {

        $response = array("status" => "error");

        if (!empty($_POST['event_id'])) {

            $product_ID = sanitize_text_field($_POST['event_id']);

            $unavailableSeats = get_post_meta($product_ID, 'fooevents_seats_unavailable_serialized', true);

            if (empty($unavailableSeats)) {

                $unavailableSeats = json_encode(array());

            }

            $response = array("status" => "success", "data" => $unavailableSeats);

        }
        
        echo json_encode($response);

        exit();

    }

    /**
     * Fetch all WooCommerce variations for seating options on product
     * 
     */
    public function fetch_all_woocommerce_variation_attributes() {
        
        global $woocommerce;
        $attributes = array();
        
        if(!empty($_POST['productID'])) {
            
            $productID = sanitize_text_field($_POST['productID']);
            $product = new WC_Product_Variable($productID);
            $variations = $product->get_children();
            foreach ($variations as $value) {
                $single_variation = new WC_Product_Variation($value);
                $varIDAttributes = array();
                array_push($varIDAttributes, $value);
                foreach ($single_variation->get_variation_attributes() as $attribute) {
                    array_push($varIDAttributes, $attribute);
                }
                array_push($attributes, $varIDAttributes);
            }
        }
        echo json_encode($attributes);
         
        exit();
    }

    /**
     * Fetch attributes for selected WooCommerce variation for seating options on product
     * 
     */
    public function fetch_selected_variation_attributes() {
        
        global $woocommerce;
        $attributes = array();
        
        if(!empty($_POST['productID'])) {
            
            $productID = sanitize_text_field($_POST['productID']);
            $product = new WC_Product_Variation($productID);
            $get_attributes = $product->get_attributes();
       
            foreach ($get_attributes as $value) {
                array_push($attributes, $value);
            }
        }
        echo json_encode($attributes);
        
         
        exit();
    }


    /**
     * Outputs the seating chart on the product screen
     * 
     * @param object $checkout
     */
    public function output_seating_fields_product($checkout) {

        global $woocommerce;
        global $post;
        global $product;

        $WooCommerceEventsViewSeatingOptions = get_post_meta($post->ID, 'WooCommerceEventsViewSeatingOptions', true);
        $WooCommerceEventsViewSeatingChart = get_post_meta($post->ID, 'WooCommerceEventsViewSeatingChart', true);
        $eventType = get_post_meta($post->ID, 'WooCommerceEventsType', true);

        if ($WooCommerceEventsViewSeatingOptions == "on" && $eventType == "seating") {
            echo '<script type="text/javascript"> if (!fooevents_seats_options_serialized) { var fooevents_seats_options_serialized = []; } </script>';
            echo '<script type="text/javascript"> if (!fooevents_seats_unavailable_serialized) { var fooevents_seats_unavailable_serialized = []; } </script>';
            echo '<script type="text/javascript"> if (!fooevents_seats_blocked_serialized) { var fooevents_seats_blocked_serialized = []; } </script>';
            echo '<script type="text/javascript"> if (!fooevents_seats_aisles_serialized) { var fooevents_seats_aisles_serialized = []; } </script>';
            echo '<script type="text/javascript"> var seatColor = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColor', true)) . '";</script>';
            echo '<script type="text/javascript"> var seatColorSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorSelected', true)) . '";</script>';
            echo '<script type="text/javascript"> var seatColorUnavailableSelected = "#' . str_replace('#', '', get_option('globalWooCommerceEventsSeatingColorUnavailableSelected', true)) . '";</script>';
            
            
            $required = true;
            
            $varID = 0;
            
            $any_variations = "";
            $fooevents_seating_data = array();


            $fooevents_seating_options_serialized = get_post_meta($post->ID,'fooevents_seating_options_serialized', true);
            $fooevents_seating_options = $this->correct_legacy_options(json_decode($fooevents_seating_options_serialized, true));
            $fooevents_seating_options_serialized = json_encode($fooevents_seating_options, true);

            $fooevents_seats_unavailable_serialized = get_post_meta($post->ID,'fooevents_seats_unavailable_serialized', true);
            $fooevents_seats_unavailable_decoded = json_decode($fooevents_seats_unavailable_serialized, true);
            
            $items = $woocommerce->cart->get_cart();
            $cart_seats = $fooevents_seats_unavailable_decoded;

            foreach($items as $item => $values) { 
            
                $fooevents_seats_unavailable_decoded = array_merge($fooevents_seats_unavailable_decoded, explode(',', $values["fooevents_seats"]));
  
            }
            
            if(empty($fooevents_seats_unavailable_serialized)) {
                $fooevents_seats_unavailable_serialized = json_encode(array());
            } 
            $fooevents_seats_unavailable = $this->correct_legacy_options_unavailable_seats($fooevents_seats_unavailable_decoded);
            $fooevents_seats_unavailable_serialized = json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);


            $fooevents_seats_blocked_serialized = get_post_meta($post->ID,'fooevents_seats_blocked_serialized', true);
            if(empty($fooevents_seats_blocked_serialized)) {
                $fooevents_seats_blocked_serialized = json_encode(array());
            } 
            $fooevents_seats_blocked = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_blocked_serialized, true));
            $fooevents_seats_blocked_serialized = json_encode($fooevents_seats_blocked, JSON_UNESCAPED_UNICODE);

            $fooevents_seats_aisles_serialized = get_post_meta($post->ID,'fooevents_seats_aisles_serialized', true);
            if(empty($fooevents_seats_aisles_serialized)) {
                $fooevents_seats_aisles_serialized = json_encode(array());
            } 
            $fooevents_seats_aisles = $this->correct_legacy_options_unavailable_seats(json_decode($fooevents_seats_aisles_serialized, true));
            $fooevents_seats_aisles_serialized = json_encode($fooevents_seats_aisles, JSON_UNESCAPED_UNICODE);
        
            
            if (!empty($fooevents_seating_options)) {
                
                foreach($fooevents_seating_options as $row_option_key => $row_object) {
                    
                    $row_key = str_replace('_option', '', $row_option_key);
                    
                    $fooevents_seating_data[$row_key . '_row_name'] = array(
                        'number_seats' => $row_object[$row_key . '_number_seats'],
                        'variation' => $row_object[$row_key . '_variations']
                    );
                    
                }


                echo '<script type="text/javascript">var fooevents_seating_data = new Object(); fooevents_seating_data["fooevents_event_' . $post->ID . '"] = JSON.parse(\'' . json_encode($fooevents_seating_data) . '\');</script>';
                echo '<script type="text/javascript">var fooevents_seats_unavailable = new Object(); fooevents_seats_unavailable["fooevents_event_' . $post->ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE) .'\');</script>';
                echo '<script type="text/javascript">var fooevents_seats_blocked = new Object(); fooevents_seats_blocked["fooevents_event_' . $post->ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_blocked) .'\');</script>';
                echo '<script type="text/javascript">var fooevents_seats_aisles = new Object(); fooevents_seats_aisles["fooevents_event_' . $post->ID . '"] = JSON.parse(\'' . json_encode($fooevents_seats_aisles) .'\');</script>';
                
                if (!empty($fooevents_seats_unavailable_serialized)) {
                    echo '<script type="text/javascript">fooevents_seats_unavailable_serialized['. $post->ID . ']  = ' . $fooevents_seats_unavailable_serialized . '</script>';
                }

                if (!empty($fooevents_seats_blocked_serialized)) {
                    echo '<script type="text/javascript">fooevents_seats_blocked_serialized['. $post->ID . ']  = ' . $fooevents_seats_blocked_serialized . '</script>';
                }

                if (!empty($fooevents_seats_aisles_serialized)) {
                    echo '<script type="text/javascript">fooevents_seats_aisles_serialized['. $post->ID . ']  = ' . $fooevents_seats_aisles_serialized . '</script>';
                }
                
                echo '<script type="text/javascript"> fooevents_seats_options_serialized[' . $post->ID . '] = ' . $fooevents_seating_options_serialized . '</script>';

                if(!empty($fooevents_seating_options) && $fooevents_seating_options != "{}") {
                    $select_values_row = array();
                    

                    foreach($fooevents_seating_options as $option_key => $option) {
                        
                        $option_ids = array_keys($option);
                        $option_values = array_values($option);
                        $option_label_output = __('Seat', 'fooevents-seating');
                        $selectLabel = $option_label_output;
                        $select_values_row[$option_ids[0]] = $option_values[0];

                    }

                    woocommerce_form_field('fooevents_seats__trans', '', "");

                    if ($WooCommerceEventsViewSeatingOptions == "on") {
                        echo "<p><a href='javascript:void(0)' name='fooevents_variation_" . $varID . "' class='fooevents_seating_chart' id='fooevents_event_id_" . $post->ID . "'><u>";
                        echo _e("Choose seats", 'fooevents-seating');
                        echo " <span>(0)</span></u></a></p><div class='clear cleafix'></div><div id='fooevents_seating_dialog' title='" . __("Seating Chart", 'fooevents-seating') . "'></div>";
                    }
    
                }
                    
            }

            if ($product->is_type( 'variable' )) {

                echo '<script type="text/javascript">var anyVariations = []; anyVariations[' . $varID . '] = "all";</script>';

            } else {

                echo '<script type="text/javascript">var anyVariations = []; anyVariations[' . $varID . '] = "none";</script>';

            }
    
        }

    }



    /**
     * Add trans fields as item data to the cart object
     * @since 1.0.0
     * @param Array $cart_item_data
     * @param int   $product_id
     * @param int   $variation_id
     * @param int   $quantity
     */
   public function add_seating_addtocart( $cart_item_data, $product_id, $variation_id, $quantity ) {

        if(!empty($_POST['fooevents_seats__trans'])) {
            $cart_item_data['fooevents_seats'] = $_POST['fooevents_seats__trans'];
        }

        return $cart_item_data;
    }





    /**
     * Converts legacy seating field options to new format
     * 
     * @param array $fooevents_seating_options
     * @return array $fooevents_seating_options  
     */
    private function correct_legacy_options($fooevents_seating_options) {

        $processed_fooevents_seating_options = array();

        $startsWithNumber = false;
        
        if(!empty($fooevents_seating_options)) {

            $x = 0;
            foreach($fooevents_seating_options as $option_key => $option) {

                $keys = array_keys($option);
                

                if (strpos($option_key, 'option') !== false) {
                
                    
                        $corrected_id = str_replace("_", "", substr($keys[0], 0, strpos($keys[0], "_row_name")));

                        /*If the corrected ID is only a number then this causes the cutsom field to be sorted incorrectly*/
                        if (preg_match('/^\d/', $corrected_id) === 1) {
                            $corrected_id = "r" . $corrected_id;
                            $startsWithNumber = true;
                        }
                        
                        if (strpos($keys[0], "_row_name") !== 0 && (substr_count($keys[0], '_') > 2 || $startsWithNumber)) {

                            $processed_fooevents_seating_options[$corrected_id][$corrected_id.'_row_name'] = $option[$keys[0]];

                        } else {
                            
                            $processed_fooevents_seating_options[$corrected_id] = $option;

                        }

                        if (strpos($keys[1], "_number_seats") !== 0 && (substr_count($keys[0], '_') > 2 || $startsWithNumber)) {

                            $processed_fooevents_seating_options[$corrected_id][$corrected_id.'_number_seats'] = $option[$keys[1]];

                        } else {

                            $processed_fooevents_seating_options[$corrected_id] = $option;

                        }

                        if (strpos($keys[2], "_variations") !== 0 && (substr_count($keys[0], '_') > 2 || $startsWithNumber)) {

                            $processed_fooevents_seating_options[$corrected_id][$corrected_id.'_variations'] = $option[$keys[2]];

                        } else {

                            $processed_fooevents_seating_options[$corrected_id] = $option;

                        }

                
                    
                    
                } else {

                    $processed_fooevents_seating_options[$option_key] = $option;

                }

            }
        
        }
        
        return $processed_fooevents_seating_options;
    
    }


    /**
     * Converts legacy unavailable seats to new format
     * 
     * @param array $fooevents_seating_options
     * @return array $fooevents_seating_options  
     */
    private function correct_legacy_options_unavailable_seats($fooevents_seats_unavailable) {

        $processed_fooevents_seats_unavailable = array();

        
        if(!empty($fooevents_seats_unavailable)) {

            
            foreach($fooevents_seats_unavailable as $unavailabe_seat) {

                $corrected_id = str_replace("_", "", substr($unavailabe_seat, 0, strpos($unavailabe_seat, "_number_seats")));
                $seat_nr = substr($unavailabe_seat, strrpos( $unavailabe_seat, '_' ) + 1);
                $corrected_id = $corrected_id . "_number_seats_" . $seat_nr;

                if (preg_match('/^\d/', $corrected_id) === 1) {
                    $corrected_id = "r" . $corrected_id;
                }

                array_push($processed_fooevents_seats_unavailable, $corrected_id);

                
            }
        
        }
        
        return $processed_fooevents_seats_unavailable;
        
    
    }


    /**
     * Adds unavailable seat to the event
     * 
     */
    function add_unavailable_seat_to_event($post_id) {
        
        $post_type = get_post_type( $post_id );
        $post_status = get_post_status( $post_id );
        $seat_row = "";
        
        if( $post_type == 'event_magic_tickets' && in_array($post_status, array('publish','draft','auto-draft','future','trash')) ) {
            $event_id = get_post_meta($post_id,"WooCommerceEventsProductID",true);
            if (empty($event_id) && isset($_POST["WooCommerceEventsEvent"])) {
                $event_id = $_POST["WooCommerceEventsEvent"];
            }
            $fooevents_seats_unavailable_serialized = get_post_meta($event_id, 'fooevents_seats_unavailable_serialized', true);
            if(empty($fooevents_seats_unavailable_serialized)) {
                $fooevents_seats_unavailable_serialized = json_encode(array());
            } 
            $fooevents_seats_unavailable = json_decode($fooevents_seats_unavailable_serialized, true);
        
            $all_post_meta = get_post_meta($post_id);

            foreach($all_post_meta as $key => $val)
            {

                if (strpos($key, 'fooevents_seat_number_') === 0) {

                    $seat_row = substr($key, strrpos( $key, '_' ) + 1 ) . '_number_seats_' . $val[0];

                }
            
            }

            if (!in_array($seat_row, $fooevents_seats_unavailable)) {

                array_push($fooevents_seats_unavailable, $seat_row);

            }
        /* hidden fields, individual custom fields, add_post_meta instead of update_post_meta,
        get_post_meta with false gives array of unavailable seats */
            $fooevents_seats_unavailable_serialized =  json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);

            update_post_meta($event_id, "fooevents_seats_unavailable_serialized", $fooevents_seats_unavailable_serialized);

        }
    }


    /**
     * Removes unavailable seat from the event
     * 
     */
    function remove_unavailable_seats_from_event($post_id, $specific_seat = 0) {

        $post_type = get_post_type( $post_id );
        $post_status = get_post_status( $post_id );
 
        if( ($post_type == 'event_magic_tickets' && in_array($post_status, array('publish','draft','future','trash'))) || $post_type == 'product' ) {
            if( $post_type == 'event_magic_tickets' ) {
                $event_id = get_post_meta($post_id,"WooCommerceEventsProductID");
                $event_id = $event_id[0];
                $all_post_meta = get_post_meta($post_id);

                foreach($all_post_meta as $key => $val) {

                    if (strpos($key, 'fooevents_seat_number_') === 0) {

                        $seat_row = substr($key, strrpos( $key, '_' ) + 1 ) . '_number_seats_' . $val[0];

                    }
                
                }
                
            }

            if( $post_type == 'product' ) {
        
                $event_id = $post_id;
                $seat_row = $specific_seat;

            }        
        
            $fooevents_seats_unavailable_serialized = get_post_meta($event_id, 'fooevents_seats_unavailable_serialized', true);
            if(empty($fooevents_seats_unavailable_serialized)) {
                $fooevents_seats_unavailable_serialized = json_encode(array());
            } 
            $fooevents_seats_unavailable = json_decode($fooevents_seats_unavailable_serialized, true);
            

            if (($key = array_search($seat_row, $fooevents_seats_unavailable)) !== false) {
                unset($fooevents_seats_unavailable[$key]);
                $fooevents_seats_unavailable = array_values($fooevents_seats_unavailable);
            }
            /* hidden fields, individual custom fields, add_post_meta instead of update_post_meta,
            get_post_meta with false gives array of unavailable seats */
            $fooevents_seats_unavailable_serialized =  json_encode($fooevents_seats_unavailable, JSON_UNESCAPED_UNICODE);
            update_post_meta($event_id, "fooevents_seats_unavailable_serialized", $fooevents_seats_unavailable_serialized);
        }



    }
    
}


$Fooevents_Seating = new Fooevents_Seating();



    


/* Add extra custom field to variations to set seating for variation required or not */
 
add_action( 'woocommerce_product_after_variable_attributes', 'fooevents_seating_add_custom_field_to_variations', 10, 3 );
 
function fooevents_seating_add_custom_field_to_variations( $loop, $variation_data, $variation ) {

$current_checkbox_value = get_post_meta($variation->ID, 'fooevents_variation_seating_required', true);
woocommerce_wp_checkbox(
    array( 
          'id' => 'fooevents_variation_seating_required['. $loop .']',
          'class' => 'fooevents_variation_seating_required',         
          'label' => __('Make seat selection required for this variation ', 'woocommerce' ), 
          'desc_tip' => true,
          'description' => __( 'If an attendee is buying a FooEvents ticket for this variation, and there exists an "Area" associated with this variation, then the attendee will be required to select a seat if this option is checked.', 'woocommerce' ),
          'value' => (!empty($current_checkbox_value) ? $current_checkbox_value : "yes"),
          
          )
        );
  
}

add_action( 'woocommerce_save_product_variation', 'fooevents_seating_save_custom_field_variations', 10, 2 );
 
function fooevents_seating_save_custom_field_variations( $variation_id, $i ) {
$fooevents_variation_seating_required =  isset( $_POST['fooevents_variation_seating_required'][$i] ) ? 'yes' : 'no';
if ( isset( $fooevents_variation_seating_required ) ) update_post_meta( $variation_id, 'fooevents_variation_seating_required', $fooevents_variation_seating_required );
}
 
// -----------------------------------------
// 3. Store custom field value into variation data
 
add_filter( 'woocommerce_available_variation', 'fooevents_seating_add_custom_field_variation_data' );
 
function fooevents_seating_add_custom_field_variation_data( $variations ) {
    
    $variations['fooevents_variation_seating_required'] = '<div class="woocommerce_custom_field"><span>' . get_post_meta( $variations[ 'variation_id' ], 'fooevents_variation_seating_required', true ) . '</span> Make seat selection required for this variation</div>';
    return $variations;
    
}

